import ts from "typescript";
import { LifeCycleEvent } from "../../Common/LifeCycle";
import { UnwrappableNode } from "../Builders/Object";
import { ActionMeta } from "./ActionMeta";
export declare type GetterSetterInfo = {
    n: string;
    c: boolean;
};
export declare type GetterSetterList = {
    [key: string]: GetterSetterInfo;
};
export declare type MetaObjectGetterSetterList = {
    [key: string]: UnwrappableNode;
};
export declare type StoreMetaObject = {
    store?: {
        className: string;
        exportName: string;
        vueBinding: string;
        module: UnwrappableNode;
    };
    actions?: {
        [key: string]: UnwrappableNode;
    };
    lifeCycleHandlers?: {
        [key: string]: LifeCycleEvent;
    };
    stateKeys?: string[];
    getters?: MetaObjectGetterSetterList;
    setters?: MetaObjectGetterSetterList;
};
export declare class StoreMeta {
    /**
     * The absolute path to the store file
     */
    absFilePath: string;
    /**
     * The relative path of the store file, for ex, if importing in /src/stores/, and our file is at /src/stores/MyStore.ts, this will be MyStore.ts
     */
    relStoreFilePath: string;
    /**
     * The relative import path from Stores/.generated/StoreLoader.ts to the store location
     * @type {string}
     */
    loaderImportPath: string;
    /**
     * The name of the store file
     */
    name: string;
    /**
     * The name of the class that extends Store
     */
    className?: string;
    /**
     * This will hold the name of our store export `export const store = new MyStore();`
     */
    exportName?: string;
    /**
     * If the dev defined `public static vueBinding = 'x';` in their store, this will hold that value
     */
    vueBinding?: string;
    /**
     * The actions which have been defined on this store
     */
    actions: {
        [key: string]: ActionMeta;
    };
    /**
     * Method name -> event type
     */
    lifeCycleHandlers: {
        [key: string]: LifeCycleEvent;
    };
    /**
     * The keys that were defined in our `get state() { return {...} }` getter
     * This will only hold root level key names
     */
    stateKeys: string[];
    /**
     * Holds the name and whether our getter is a computed property, for all js getters defined on the store
     */
    getters: GetterSetterList;
    /**
     * Holds the name and whether our setter is a computed property, for all js setters defined on the store
     */
    setters: GetterSetterList;
    /**
     * The AST of our state getter object, this is mainly used to do some checks/validation later in the processing
     */
    stateObj: ts.ObjectLiteralExpression;
    metaObject: StoreMetaObject;
    constructor(file: ts.SourceFile);
    /**
     * Ensure that we have everything we need to process this as a store and register it with the loader
     */
    isValid(): boolean;
    /**
     * Apply any final modifications we need to do before outputting the store
     */
    finalize(): this;
    /**
     * Ensure that we actually have a vue binding specified
     *
     * Either use that static vue binding property defined on the store class
     * Or fallback to creating one from the class name of the store
     */
    private formatVueBinding;
    /**
     * Set the state object and extract all keys from the object
     */
    setStateObject(stateObj: ts.ObjectLiteralExpression): void;
    /**
     * Add an action definition to our store
     */
    addAction(action: ActionMeta): void;
    /**
     * Add a getter definition to our store
     */
    addGetter(member: ts.GetAccessorDeclaration): void;
    /**
     * Add a setter definition to our store
     *
     * We also need to check if this setter has a corresponding getter
     * If it does, and it has the @Computed decorator, we need to mark this setter as
     * computed also, this will allow the vue lib to create a writable computed property
     */
    addSetter(member: ts.SetAccessorDeclaration): void;
    /**
     * Store that X method name handles x lifecycle event
     */
    addLifeCycleHandler(method: ts.MethodDeclaration, lifeCycleEvent: LifeCycleEvent): void;
    process(declaration: ts.ClassDeclaration): void;
    /**
     * Create the meta object which will be stored in the generated StoreLoader.ts file
     */
    toMetaObject(): StoreMetaObject;
}
