"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isStateGetterNode = exports.isVueBinding = exports.extendsStore = void 0;
const typescript_1 = __importDefault(require("typescript"));
const ErrorMessages_1 = require("../ErrorMessages");
const ProcessingContext_1 = require("../Utils/ProcessingContext");
const Modifiers_1 = require("./Modifiers");
/**
 * Check if our class extends the main Store class
 * Ex:
 *
 * export class MyStore extends Store<MyStore, IMyStoreState>
 *
 * @param {ts.ClassDeclaration} classDeclaration
 * @returns {boolean | undefined}
 */
function extendsStore(classDeclaration) {
    var _a;
    if (!typescript_1.default.isClassDeclaration(classDeclaration))
        return false;
    return (_a = classDeclaration.heritageClauses) === null || _a === void 0 ? void 0 : _a.some(h => {
        if (h.token !== typescript_1.default.SyntaxKind.ExtendsKeyword)
            return false;
        return h.types.some(t => {
            var _a, _b;
            if (!typescript_1.default.isExpressionWithTypeArguments(t))
                return false;
            if (!typescript_1.default.isCallExpression(t.expression))
                return false;
            const callExpression = t.expression;
            if (!typescript_1.default.isIdentifier(callExpression.expression))
                return false;
            return ((_a = callExpression.expression.text) === null || _a === void 0 ? void 0 : _a.toLowerCase()) === "store" || ((_b = callExpression.expression.text) === null || _b === void 0 ? void 0 : _b.toLowerCase()) === "basestore";
        });
    });
}
exports.extendsStore = extendsStore;
/**
 * Check if our class member is the:
 * `public static vueBinding = 'x';` statement
 *
 * If it is, return true & the value of this property
 *
 * @param {ts.ClassElement} member
 * @returns {[is: boolean, binding: string]}
 */
function isVueBinding(member) {
    var _a;
    if (!typescript_1.default.isPropertyDeclaration(member))
        return [false, undefined];
    if (!((_a = member.modifiers) === null || _a === void 0 ? void 0 : _a.some(m => m.kind === typescript_1.default.SyntaxKind.StaticKeyword)))
        return [false, undefined];
    if (!typescript_1.default.isIdentifier(member.name))
        return [false, undefined];
    if (member.name.text !== 'vueBinding')
        return [false, undefined];
    if (!typescript_1.default.isStringLiteral(member.initializer))
        return [false, undefined];
    return [member.name.text === "vueBinding", member.initializer.text];
}
exports.isVueBinding = isVueBinding;
function isStateGetterNode(member) {
    const result = [false, undefined];
    if (typescript_1.default.isPropertyDeclaration(member))
        return result;
    if (!typescript_1.default.isIdentifier(member.name))
        return result;
    if (member.name.text !== 'state')
        return result;
    // Ensure our state getter is actually a getter
    if (!typescript_1.default.isGetAccessor(member)) {
        ProcessingContext_1.ProcessingContext.linter.error(ErrorMessages_1.errorMessages.stateGetter.mustBeGetter(), member);
        return result;
    }
    // Ensure it's defined as public or private
    if (!(0, Modifiers_1.isPublicOrPrivate)(member.modifiers)) {
        ProcessingContext_1.ProcessingContext.linter.error(ErrorMessages_1.errorMessages.stateGetter.publicOrPrivate(), member);
        return result;
    }
    // Ensure it's not static
    if ((0, Modifiers_1.isStatic)(member.modifiers)) {
        ProcessingContext_1.ProcessingContext.linter.error(ErrorMessages_1.errorMessages.stateGetter.nonStatic(), member);
        return result;
    }
    const body = member.body;
    if (!typescript_1.default.isBlock(body))
        return result;
    const statements = body.statements;
    if (statements.length !== 1) {
        ProcessingContext_1.ProcessingContext.linter.error(ErrorMessages_1.errorMessages.stateGetter.invalidBody(), member);
        return result;
    }
    const statement = statements[0];
    if (!typescript_1.default.isReturnStatement(statement)) {
        ProcessingContext_1.ProcessingContext.linter.error(ErrorMessages_1.errorMessages.stateGetter.invalidBody(), member);
        return result;
    }
    if (!typescript_1.default.isObjectLiteralExpression(statement.expression)) {
        ProcessingContext_1.ProcessingContext.linter.error(ErrorMessages_1.errorMessages.stateGetter.invalidBody(), member);
        return result;
    }
    return [true, statement.expression];
}
exports.isStateGetterNode = isStateGetterNode;
//# sourceMappingURL=Classes.js.map