"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Context = void 0;
const fs_jetpack_1 = __importDefault(require("fs-jetpack"));
const typescript_1 = __importDefault(require("typescript"));
const crypto_1 = __importDefault(require("crypto"));
const Classes_1 = require("./AstHelpers/Classes");
const Exports_1 = require("./AstHelpers/Exports");
const StoreLoader_1 = require("./Builders/StoreLoader");
const VueDtsFile_1 = require("./Builders/VueDtsFile");
const Linting_1 = require("./Linting");
const Logger_1 = require("./Logger");
const StoreMeta_1 = require("./Meta/StoreMeta");
const PluginConfig_1 = require("./PluginConfig");
const TS_1 = require("./TS");
const ProcessingContext_1 = require("./Utils/ProcessingContext");
const Timed_1 = require("./Utils/Timed");
const { factory } = typescript_1.default;
class Context {
    constructor() {
        /**
         * When we first load our stores, we'll take a hash of the content
         * Later on this hash will be used when there is a change, mainly to
         * determine if we need to do some file writes, when writing stores.d.ts,
         * even if contents are the same, this makes vite do a full page reload...
         * @type {Map<string, string>}
         */
        Object.defineProperty(this, "storeFileHashes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Map()
        });
        /**
         * All of the store modules in our project
         *
         * @type {{module: ts.SourceFile, store: StoreMeta}[]}
         */
        Object.defineProperty(this, "modules", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        /**
         * All of our store meta classes which hold information on all our stores.
         *
         * @type {StoreMeta[]}
         */
        Object.defineProperty(this, "stores", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
    }
    init() {
        Timed_1.Timed.start('generator.init');
        PluginConfig_1.PluginConfig.setStoreFilePaths();
        TS_1.TS.setup();
        this.reloadModules(false);
        this.loadGeneratedFileHash(PluginConfig_1.PluginConfig.storesFileName);
        this.loadGeneratedFileHash(PluginConfig_1.PluginConfig.storeLoaderFile);
        Timed_1.Timed.end('generator.init');
    }
    /**
     * Load all of our store modules and create a "store meta" class.
     */
    reloadModules(reloadFilePaths = true) {
        if (reloadFilePaths) {
            PluginConfig_1.PluginConfig.setStoreFilePaths();
        }
        this.modules = TS_1.TS.program.getSourceFiles()
            .reduce((acc, source) => {
            const filePath = typescript_1.default.sys.resolvePath(source.fileName);
            if (!PluginConfig_1.PluginConfig.isStoreFilePath(filePath)) {
                return acc;
            }
            if (source.isDeclarationFile) {
                return acc;
            }
            acc.push({
                module: source,
                store: new StoreMeta_1.StoreMeta(source),
            });
            this.addFileHash(filePath, source.getFullText());
            return acc;
        }, []);
    }
    /**
     * Create a file hash for the specified file path + content
     *
     * This will be used to prevent over-writing to the FS when the content is the same
     *
     * @param {string} filePath
     * @param {string} content
     * @returns {boolean}
     * @private
     */
    addFileHash(filePath, content) {
        const hash = crypto_1.default.createHash('sha256').update(content).digest('hex');
        if (this.storeFileHashes.has(filePath)) {
            const oldHash = this.storeFileHashes.get(filePath);
            if (oldHash === hash) {
                return false;
            }
            this.storeFileHashes.set(filePath, hash);
            (0, Logger_1.warnLog)("Store file hash is different: " + filePath);
            return true;
        }
        this.storeFileHashes.set(filePath, hash);
        return true;
    }
    /**
     * Quick helper method to setup file hashes for generated files
     *
     * @param {string} file
     * @private
     */
    loadGeneratedFileHash(file) {
        if (PluginConfig_1.PluginConfig.generatedDir.exists(file)) {
            const content = PluginConfig_1.PluginConfig.generatedDir.read(file);
            this.addFileHash(PluginConfig_1.PluginConfig.generatedDir.path(file), content);
        }
    }
    /**
     * MAKE SURE TO PASS A FULL ABSOLUTE PATH TO THIS METHOD!
     *
     * @param {string} filePath
     * @returns {boolean}
     * @private
     */
    fileHasChanged(filePath) {
        const fPath = fs_jetpack_1.default.path(filePath);
        if (!this.storeFileHashes.has(fPath)) {
            return true;
        }
        const content = fs_jetpack_1.default.read(fPath);
        const hash = crypto_1.default.createHash('sha256').update(content).digest('hex');
        return this.storeFileHashes.get(filePath) !== hash;
    }
    process() {
        Linting_1.Linting.reset();
        this.stores = [];
        for (const { module, store } of this.modules) {
            ProcessingContext_1.ProcessingContext.setCurrent(module, store);
            Timed_1.Timed.start('generator.processModule');
            const outStore = this.processModule(module, store);
            Timed_1.Timed.end('generator.processModule');
            if (!outStore)
                continue;
            this.stores.push(outStore.finalize());
        }
        Linting_1.Linting.print();
    }
    writeFiles() {
        Timed_1.Timed.start('generator.writeFiles');
        this.writeStoreDeclaration();
        Timed_1.Timed.end('generator.writeFiles');
    }
    writeStoreDeclaration() {
        var _a;
        const storeCount = (_a = this.stores) === null || _a === void 0 ? void 0 : _a.length;
        if (!storeCount) {
            (0, Logger_1.warnLog)('No stores located... skipping code-gen.');
            return;
        }
        (0, Logger_1.infoLog)('Located', Logger_1.colors.ResetWrap(storeCount), 'stores:');
        (0, Logger_1.basicLog)(this.stores.map(s => "   - " + Logger_1.colors.ResetWrap(s.className)).join('\n'));
        const vueDtsFile = (0, VueDtsFile_1.createVueDtsFile)(this.stores);
        const { didWrite, filePath } = this.writeTsFile(PluginConfig_1.PluginConfig.vueDtsPath, vueDtsFile);
        if (didWrite) {
            (0, Logger_1.successLog)('Generated stores vue declaration file at:', Logger_1.colors.ResetWrap(PluginConfig_1.PluginConfig.generatedDir.path(filePath).replace(PluginConfig_1.PluginConfig.projectRoot, '')));
        }
        const loaderResult = this.writeTsFile(PluginConfig_1.PluginConfig.generatedDir.path(PluginConfig_1.PluginConfig.storeLoaderFile), (0, StoreLoader_1.createStoreLoaderModule)(this.stores));
        if (loaderResult.didWrite) {
            (0, Logger_1.successLog)('Generated store loader file at:', Logger_1.colors.ResetWrap(PluginConfig_1.PluginConfig.generatedDir.path(loaderResult.filePath).replace(PluginConfig_1.PluginConfig.projectRoot, '')));
        }
    }
    rebuild() {
        PluginConfig_1.PluginConfig.setStoreFilePaths();
        TS_1.TS.rebuild();
        this.reloadModules(false);
        this.process();
        this.writeFiles();
    }
    processModule(module, store) {
        for (let statement of module.statements) {
            // Search for our `export myStoreName = new MyStore();` statement and extract `myStoreName`
            const [isExport, storeName] = (0, Exports_1.isExportConst)(statement);
            if (isExport) {
                store.exportName = storeName;
            }
            // Search for our `export class MyStore extends Store<MyStore, IMyStoreState> {` statement and extract `MyStore`
            if (!typescript_1.default.isClassDeclaration(statement))
                continue;
            // Ensure our class extends Store
            if (!(0, Classes_1.extendsStore)(statement))
                continue;
            if (!typescript_1.default.isIdentifier(statement.name))
                continue;
            ProcessingContext_1.ProcessingContext.processingStoreClass(statement, () => {
                store.process(statement);
            });
        }
        ProcessingContext_1.ProcessingContext.validate();
        return store.isValid() ? store : undefined;
    }
    isStoreFile(path) {
        return this.stores.some(s => s.absFilePath === path);
    }
    getStoreByFilePath(path) {
        return this.stores.find(s => s.absFilePath === path);
    }
    writeTsFile(filePath, nodes) {
        const result = { didWrite: false, filePath: filePath };
        const sourceFile = typescript_1.default.createSourceFile(filePath, '', PluginConfig_1.PluginConfig.tsConfig.options.target, false, typescript_1.default.ScriptKind.TS);
        const source = TS_1.TS.printer.printList(typescript_1.default.ListFormat.MultiLine, factory.createNodeArray(nodes), sourceFile);
        result.didWrite = this.writeFile(filePath, source);
        return result;
    }
    writeFile(filePath, contents) {
        if (!this.addFileHash(filePath, contents)) {
            return false;
        }
        fs_jetpack_1.default.write(filePath, contents);
        return true;
    }
    static printNode(node) {
        try {
            const sourceFile = typescript_1.default.createSourceFile('temp.ts', '', PluginConfig_1.PluginConfig.tsConfig.options.target, false, typescript_1.default.ScriptKind.TS);
            (0, Logger_1.infoLog)('Printed node > \n' + TS_1.TS.printer.printNode(typescript_1.default.EmitHint.Unspecified, node, sourceFile));
        }
        catch (error) {
            (0, Logger_1.errorLog)("Failed to print node", error);
        }
    }
}
exports.Context = Context;
//# sourceMappingURL=Generator.js.map