"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ActionMeta = void 0;
const LifeCycle_1 = require("../../Common/LifeCycle");
const Signature_1 = require("../AstHelpers/Signature");
const ErrorMessages_1 = require("../ErrorMessages");
const ProcessingContext_1 = require("../Utils/ProcessingContext");
class ActionMeta {
    constructor(method) {
        /**
         * The name of the method
         */
        Object.defineProperty(this, "name", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * The parameters of the method
         */
        Object.defineProperty(this, "params", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        /**
         * The decorators defined on the method
         */
        Object.defineProperty(this, "decorators", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
        Object.defineProperty(this, "decoratorMeta", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
        /**
         * If we defined a lifecycle decorator on the method, we'll return it here
         */
        Object.defineProperty(this, "lifeCycleEventHandler", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * Typescript AST for the method
         */
        Object.defineProperty(this, "method", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.method = method;
    }
    prepare() {
        const methodMeta = (0, Signature_1.extractSignatureMeta)(this.method);
        this.name = methodMeta.name;
        this.params = methodMeta.parameters;
        const decorators = (0, Signature_1.extractDecoratorsFromSignature)(this.method, (decorator, info) => {
            return (0, LifeCycle_1.isLifeCycleEvent)(info.name) || info.name === 'On';
        });
        for (let decorator of decorators) {
            this.setDecorator(decorator);
        }
    }
    isValid() {
        // If this action is an event handler, we don't want to store it in the actions list
        // We'll use the store's lifeCycleHandlers instead
        if (this.lifeCycleEventHandler) {
            return false;
        }
        return true;
    }
    toJSON() {
        return {
            name: this.name,
            params: this.params,
            lifeCycleEventHandler: this.lifeCycleEventHandler,
        };
    }
    setDecorator(decorator) {
        if ((0, LifeCycle_1.isLifeCycleEvent)(decorator.name)) {
            if (this.lifeCycleEventHandler) {
                ProcessingContext_1.ProcessingContext.linter.error(ErrorMessages_1.errorMessages.lifecycle.multipleLifeCycleHandlersDefinedForMethod(this.name, this.lifeCycleEventHandler), this.method);
                return;
            }
            ProcessingContext_1.ProcessingContext.store.addLifeCycleHandler(this.method, decorator.name);
            this.lifeCycleEventHandler = decorator.name;
            this.defineDecorator(decorator);
            return;
        }
        this.defineDecorator(decorator);
    }
    defineDecorator(decorator) {
        this.decorators[decorator.name] = decorator;
        this.decoratorMeta[decorator.name] = {
            n: decorator.name,
            p: decorator.parameters.map(p => ({ n: p.name, v: p.value })),
        };
    }
    toMetaObject() {
        return {
            n: this.name,
            p: this.params.map(p => ({ n: p.name, t: p.type, dv: p.defaultValue })),
            d: this.decoratorMeta,
            h: this.lifeCycleEventHandler,
        };
    }
}
exports.ActionMeta = ActionMeta;
//# sourceMappingURL=ActionMeta.js.map