"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StoreMeta = void 0;
const path_1 = __importDefault(require("path"));
const typescript_1 = __importDefault(require("typescript"));
const Common_1 = require("../../Common");
const Classes_1 = require("../AstHelpers/Classes");
const Modifiers_1 = require("../AstHelpers/Modifiers");
const Imports_1 = require("../Builders/Imports");
const Object_1 = require("../Builders/Object");
const ErrorMessages_1 = require("../ErrorMessages");
const Logger_1 = require("../Logger");
const PluginConfig_1 = require("../PluginConfig");
const ProcessingContext_1 = require("../Utils/ProcessingContext");
const ActionMeta_1 = require("./ActionMeta");
const Paths_1 = require("../Utils/Paths");
class StoreMeta {
    constructor(file) {
        /**
         * The absolute path to the store file
         */
        Object.defineProperty(this, "absFilePath", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * The relative path of the store file, for ex, if importing in /src/stores/, and our file is at /src/stores/MyStore.ts, this will be MyStore.ts
         */
        Object.defineProperty(this, "relStoreFilePath", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * The relative import path from Stores/.generated/StoreLoader.ts to the store location
         * @type {string}
         */
        Object.defineProperty(this, "loaderImportPath", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * The name of the store file
         */
        Object.defineProperty(this, "name", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * The name of the class that extends Store
         */
        Object.defineProperty(this, "className", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * This will hold the name of our store export `export const store = new MyStore();`
         */
        Object.defineProperty(this, "exportName", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * If the dev defined `public static vueBinding = 'x';` in their store, this will hold that value
         */
        Object.defineProperty(this, "vueBinding", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * The actions which have been defined on this store
         */
        Object.defineProperty(this, "actions", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
        /**
         * Method name -> event type
         */
        Object.defineProperty(this, "lifeCycleHandlers", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
        /**
         * The keys that were defined in our `get state() { return {...} }` getter
         * This will only hold root level key names
         */
        Object.defineProperty(this, "stateKeys", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        /**
         * Holds the name and whether our getter is a computed property, for all js getters defined on the store
         */
        Object.defineProperty(this, "getters", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
        /**
         * Holds the name and whether our setter is a computed property, for all js setters defined on the store
         */
        Object.defineProperty(this, "setters", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
        /**
         * The AST of our state getter object, this is mainly used to do some checks/validation later in the processing
         */
        Object.defineProperty(this, "stateObj", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "metaObject", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {
                actions: {},
                lifeCycleHandlers: {},
                stateKeys: [],
                getters: {},
                setters: {},
            }
        });
        this.absFilePath = file.fileName;
        this.name = path_1.default.basename(file.fileName);
        this.relStoreFilePath = (0, Paths_1.formatImport)(path_1.default.relative(PluginConfig_1.PluginConfig.generatedDir.path(), file.fileName));
        this.loaderImportPath = this.relStoreFilePath;
    }
    /**
     * Ensure that we have everything we need to process this as a store and register it with the loader
     */
    isValid() {
        return this.className !== undefined && this.exportName !== undefined;
    }
    /**
     * Apply any final modifications we need to do before outputting the store
     */
    finalize() {
        this.formatVueBinding();
        // debugLog(`Found getters: ${this.getters?.map(g => g.n).join(', ')} in ${this.className}`);
        return this;
    }
    /**
     * Ensure that we actually have a vue binding specified
     *
     * Either use that static vue binding property defined on the store class
     * Or fallback to creating one from the class name of the store
     */
    formatVueBinding() {
        if (this.vueBinding) {
            if (!this.vueBinding.startsWith("$")) {
                this.vueBinding = "$" + this.vueBinding;
            }
            return;
        }
        this.vueBinding = (0, Common_1.formatVueBindingName)(undefined, this.className);
    }
    /**
     * Set the state object and extract all keys from the object
     */
    setStateObject(stateObj) {
        var _a;
        this.stateObj = stateObj;
        for (let property of stateObj.properties) {
            if (typescript_1.default.isPropertyAssignment(property) && (property === null || property === void 0 ? void 0 : property.name)) {
                this.stateKeys.push((_a = property.name) === null || _a === void 0 ? void 0 : _a.escapedText);
            }
        }
        this.metaObject.stateKeys = this.stateKeys;
        // debugLog(`Found state keys ${this.stateKeys.join(", ")} in ${this.className}`);
    }
    /**
     * Add an action definition to our store
     */
    addAction(action) {
        action.prepare();
        if (!action.isValid()) {
            return;
        }
        this.actions[action.name] = action;
        this.metaObject.actions[action.name] = (0, Object_1.unwrappableNode)((0, Object_1.newClassInstance)("StoreMetaActionData", [(0, Object_1.createNodeFromValue)(this.actions[action.name].toMetaObject())]));
    }
    /**
     * Add a getter definition to our store
     */
    addGetter(member) {
        if (member.name.getText() === "state") {
            return;
        }
        const getter = {
            n: member.name.getText(),
            c: (0, Modifiers_1.hasDecorator)(member, "Computed"),
        };
        this.getters[getter.n] = getter;
        this.metaObject.getters[getter.n] = (0, Object_1.unwrappableNode)((0, Object_1.newClassInstance)("StoreMetaGetterSetterData", [(0, Object_1.createNodeFromValue)("getter"), (0, Object_1.createNodeFromValue)(getter)]));
    }
    /**
     * Add a setter definition to our store
     *
     * We also need to check if this setter has a corresponding getter
     * If it does, and it has the @Computed decorator, we need to mark this setter as
     * computed also, this will allow the vue lib to create a writable computed property
     */
    addSetter(member) {
        const getterSymbol = member === null || member === void 0 ? void 0 : member.symbol;
        if (!getterSymbol) {
            (0, Logger_1.errorLog)(`Could not find symbol for setter ${member.name.getText()} in ${this.className}. Please create an issue on the plugin's github`);
            return;
        }
        const declarations = getterSymbol.declarations;
        if (!declarations) {
            (0, Logger_1.errorLog)(`Could not find declarations for setter ${member.name.getText()} in ${this.className}. Please create an issue on the plugin's github`);
            return;
        }
        const setterInfo = {
            n: member.name.getText(),
            c: false,
        };
        const getter = declarations.find(d => typescript_1.default.isGetAccessorDeclaration(d));
        // We can only make it a computed property if there is a corresponding getter
        if (getter) {
            setterInfo.c = (0, Modifiers_1.hasDecorator)(getter, "Computed");
        }
        this.setters[setterInfo.n] = setterInfo;
        this.metaObject.setters[setterInfo.n] = (0, Object_1.unwrappableNode)((0, Object_1.newClassInstance)("StoreMetaGetterSetterData", [(0, Object_1.createNodeFromValue)("setter"), (0, Object_1.createNodeFromValue)(setterInfo)]));
    }
    /**
     * Store that X method name handles x lifecycle event
     */
    addLifeCycleHandler(method, lifeCycleEvent) {
        if (Object.values(this.lifeCycleHandlers).includes(lifeCycleEvent)) {
            ProcessingContext_1.ProcessingContext.linter.error(ErrorMessages_1.errorMessages.lifecycle.multipleLifeCycleHandlersDefinedForEvent(ProcessingContext_1.ProcessingContext.action.name), method);
            return;
        }
        this.lifeCycleHandlers[method.name.getText()] = lifeCycleEvent;
        this.metaObject.lifeCycleHandlers = this.lifeCycleHandlers;
    }
    process(declaration) {
        // Store the class name for our store
        this.className = declaration.name.text;
        // Now we'll process the members of the class
        // We need to extract Action meta & the vue binding(if one is defined)
        for (let member of declaration.members) {
            // Now we have to look for the `public static vueBinding = 'x';`
            // statement and pull out it's value.
            const [isBinding, vueBinding] = (0, Classes_1.isVueBinding)(member);
            if (isBinding) {
                this.vueBinding = vueBinding;
                continue;
            }
            // Check if our member is an action, if it is we'll store some meta for it
            if (typescript_1.default.isMethodDeclaration(member) && typescript_1.default.isIdentifier(member.name)) {
                const action = new ActionMeta_1.ActionMeta(member);
                ProcessingContext_1.ProcessingContext.processingAction(action, () => {
                    this.addAction(action);
                });
                continue;
            }
            const [isStateGetter, stateObj] = (0, Classes_1.isStateGetterNode)(member);
            if (isStateGetter && !this.stateObj) {
                this.setStateObject(stateObj);
                continue;
            }
            if (typescript_1.default.isGetAccessor(member) && typescript_1.default.isIdentifier(member.name)) {
                this.addGetter(member);
                continue;
            }
            if (typescript_1.default.isSetAccessor(member) && typescript_1.default.isIdentifier(member.name)) {
                this.addSetter(member);
            }
        }
    }
    /**
     * Create the meta object which will be stored in the generated StoreLoader.ts file
     */
    toMetaObject() {
        this.metaObject.store = {
            className: this.className,
            exportName: this.exportName,
            vueBinding: this.vueBinding,
            module: (0, Object_1.unwrappableNode)((0, Imports_1.createLazyImportGlobNode)(this.loaderImportPath))
        };
        return this.metaObject;
    }
}
exports.StoreMeta = StoreMeta;
//# sourceMappingURL=StoreMeta.js.map